#!/usr/bin/perl

# NOTE: This script supports IBM N-series filers running Data Ontap releases == 7.x
$myVersion = "1.1";

$rshCommand[0] = "rsh -l :";
$rshCommand[1] = "rsh -l :";
$versionCommand = "version";
$licenseCommand = "\"priv set -q advanced; registry walk options.license\"";
$optionsCommand = "options";
$ifconfigCommand = "\"priv set -q advanced; registry walk status.if\"";
$cfStatusCommand = "cf status";
$hostnameCommand = "\"priv set -q advanced; hostname\"";
$cfPartnerCommand = "cf partner";
$fcpcfModeCommand = "fcp show cfmode";

$script = $ENV{'SCRIPT_NAME'};
$ActAsCgi = defined( $ENV{ 'GATEWAY_INTERFACE' });

#main -- parse input data and call subroutines to do the real work.
if ( $ActAsCgi )
{
# run as cgi script
  # Send error messages to the user, not system log
  open(STDERR,'<&STDOUT');  $| = 1;
  print "Content-type: text/html\n\n";
  print "<HTML><HEAD><TITLE>Data ONTAP&#153 Cluster Configuration Checker</TITLE></HEAD></HTML>";

  use CGI;
  &CGI::ReadParse(*input);

  if ($input{'button'}) {
    if ($input{'button'} =~ "About") {
      printAbout();
    }
    if ($input{'button'} =~ "Help") {
      printHelp();
    }
    if ($input{'button'} =~ "Check Configuration") {
      if ($input{'filer1'} && $input{'filer2'} && 
	  ($input{'filer1'} ne $input{'filer2'})) {
	if ($input{'debug'} =~ "Debug"){
	  $DEBUG = 1;
	}
	else {
	  $DEBUG = 0;
	}
	$filer[0] = $input{'filer1'};
	if ($input{'uid1'}) {
	  $rshCommand[0] =~ s/:/$input{'uid1'}:/;
	}
	if ($input{'passwd1'}) {
	  $rshCommand[0] =~ s/:/:$input{'passwd1'}/;
	}
	$filer[1] = $input{'filer2'};
	if ($input{'uid2'}) {
	  $rshCommand[1] =~ s/:/$input{'uid2'}:/;
	}
	if ($input{'passwd2'}) {
	  $rshCommand[1] =~ s/:/:$input{'passwd2'}/;
	}
	print "<H1>Checking configuration of cluster: $filer[0] and $filer[1]</H1>";
	if (&checkLogins == 0) {
	  if (&checkOSversions == 0) {
	    if (&checkLicenseMismatch == 0) {
	      if (&checkClusterIdentity == 0) {
		if (&checkCfStatus == 0) {
		  if (&checkFCPcfmodeMismatch == 0) {
		    if (&checkOptionsMismatch == 0) {
		      &checkNetworkConfig;
		    }
		  }
		}
	      }
	    }
	  }
	}
	print "Done.";
      }
      else {
	&showForm;
      }
    }
  }
  else {
    &showForm;
  }
}

else {
# run as a command line perl script
  if ($#ARGV != 1 && $#ARGV != 2) {
    printUsage();
    exit;
  }
  else {
    if ($#ARGV == 2) {
      if ($ARGV[2] ne "verbose") {
        printUsage();
        exit;
      }
      else {
        $DEBUG = 1;
      }
    }
    else {
      $DEBUG = 0;
    }
    $filer[0] = $ARGV[0];
    print "$filer[0] rsh login: ";
    chomp($uid1 = <STDIN>);
    $rshCommand[0] =~ s/:/$uid1:/;
    system "stty -echo";
    print "Password: ";
    chomp($passwd1 = <STDIN>);
    print "\n";
    system "stty echo";
    $rshCommand[0] =~ s/:/:$passwd1/;
    $filer[1] = $ARGV[1];
    print "$filer[1] rsh login: ";
    chomp($uid2 = <STDIN>);
    $rshCommand[1] =~ s/:/$uid2:/;
    system "stty -echo";
    print "Password: ";
    chomp($passwd2 = <STDIN>);
    print "\n";
    system "stty echo";
    $rshCommand[1] =~ s/:/:$passwd2/;

    if (&checkLogins == 0) {
      if (&checkOSversions == 0) {
        if (&checkLicenseMismatch == 0) {
          if (&checkClusterIdentity == 0) {
            if (&checkCfStatus == 0) {
	      if (&checkFCPcfmodeMismatch == 0) {
	        if (&checkOptionsMismatch == 0) {
	          &checkNetworkConfig;
		}
              }
            }
          }
        }
      }
    }
    print "Done.\n";
  }
}


#present html form for submitting names of cluster nodes
sub showForm {
  print <<EOF;
  <CENTER>
  <H1>Data ONTAP&#153 Cluster Configuration Checker</H1>
  <FORM METHOD="POST"
  ACTION=$script>
  <INPUT TYPE=submit NAME=button VALUE="About">
  <INPUT TYPE=submit NAME=button VALUE="Help">
  <PRE>
<H3>First Cluster Node</H3>
name or IP address <INPUT NAME=filer1>
rsh login <INPUT NAME=uid1 VALUE=\"root\">
password <INPUT TYPE="password" NAME=passwd1>
    <P>
<H3>Second Cluster Node</H3>
name or IP address <INPUT NAME=filer2>
rsh login <INPUT NAME=uid2 VALUE=\"root\">
password <INPUT TYPE="password" NAME=passwd2>
    <P>
Output Verbosity <SELECT NAME=debug>
  <OPTION>Normal<OPTION>Debug</SELECT>
  </PRE>
    <INPUT TYPE=submit NAME=button VALUE="Check Configuration">
  </FORM>
  </CENTER>
EOF
}


# Can we rsh into the filers?
sub checkLogins {
  printHeading("Checking rsh logins ...");
  for ($fn = 0; $fn < 2; $fn++) {
    $foundFilerData = 0;
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $versionCommand |") ) {
      print "Can't run $versionCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      while (<FILERDATA>) {
        $foundFilerData ++;
      }
      close FILERDATA;
    }
    if (!$foundFilerData) {
      print "Cannot rsh into $filer[$fn]";
      printNewLine(1);
      return 1;
    }
  }
  if ($DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}


# look for OS version mismatch
sub checkOSversions {
  $foundError = 0;
  printHeading("Checking OS versions...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $versionCommand |") ) {
      print "Can't run $versionCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      $lineNumber = 0;
      while (<FILERDATA>) {
	if ($DEBUG) { print; printNewLine(1);}

        # Can we work with this OS ?
        if ($lineNumber == 0 && ! m/^Data ONTAP.*$/) {
          print "$filer[$fn] is not an IBM N-series filer";
          printNewLine(1);
          return 1;
        } 
        if ($lineNumber == 0 && $_ =~ /^Data ONTAP Release (\d+)\.(\d+)\.(\d+).*$/) {
          if ($1 < 7) {
            print "cf-config-check.cgi does not support Data ONTAP Releases < 7.x";
            printNewLine(1);
            print "$filer[$fn] is running $_";
            printNewLine(1);
            return 1;
          }
#
# Logic for version == 5 removed  
# Pre 7.x releases N/A to IBM N-series
#
          if ($1 > 7) {
            print "cf-config-check.cgi version $myVersion does not support ";
            print "Data ONTAP Releases > 7.x";
            printNewLine(1);
            print "$filer[$fn] is running $_";
            printNewLine(1);
            print "Please download the latest version of cf-config-check.cgi from";
            printNewLine(1);
            if ($ActAsCgi) {
              print "<a href=\"http://www.ibm.com/storage/support/nas/\">http://www.ibm.com/storage/support/nas</a>";
            }
            else {
              print "http://www.ibm.com/storage/support/nas/";
            }
            printNewLine(1);
            return 1;
          }
        }
     
        # Do the OS versions match ?
        if ($fn == 0) {
	  $versionString[$lineNumber++] = $_;
	}
	else {
	  if ($versionString[$lineNumber++] ne $_) {
	    print "WARNING: OS versions do not match";
            printNewLine(1);
            $foundError = 1;
	  }
	}
      }
    }
    close FILERDATA;
    if ($DEBUG) { printNewLine(1); };
  }
  if ($foundError == 0 && $DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}

#
# rcToggleBasicCmds subroutine removed 
# Pre 6.0 releases N/A to IBM N-series
#

# look for license mismatch
sub checkLicenseMismatch {
  $foundError = 0;
  printHeading("Checking licenses...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $licenseCommand |") ) {
      print "Can't run $licenseCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      $clusterLicensed = 0;
      $listIndex = 0;
      while (<FILERDATA>) {
	if (! m/=off$/) {
	  s/options\.license.//;
	  s/=[A-Z]+//;
	  s/\n|\r/ /;
          if ($_ =~ "cluster") {
	    $clusterLicensed = 1;
          }
	  if ($fn == 0) {
	    $licenseList[$listIndex++] = $_;
	  }
	  else {
	    $listIndex = 0;
	    $foundLicense = 0;
	    while ($listIndex <= $#licenseList && $foundLicense == 0) {
	      if ($licenseList[$listIndex] eq $_) {
		$licenseList[$listIndex] = "ERASED";
		$foundLicense = 1;
	      }
	      $listIndex++;
	    }
	    if ($foundLicense == 0) {
	      $foundError = 1;
	      print "$_: licensed on $filer[1], not licensed on $filer[0].";
              printNewLine(1);
	    }
	  }
	  if ($DEBUG) { print; printNewLine(1); }
	}
      }
      if ($clusterLicensed == 0) {
	$foundError = 1;
        print "Cluster not licensed on $filer[$fn].";
        printNewLine(1);
      }
    }
    close FILERDATA;
    if ($DEBUG) { printNewLine(1); }
  }
  for ($listIndex = 0; $listIndex <= $#licenseList; $listIndex++) {
    if ($licenseList[$listIndex] ne "ERASED") {
      $foundError = 1;
      print "$licenseList[$listIndex]: licensed on $filer[0], not licensed on $filer[1].";
      printNewLine(1);
    }
  }
  if ($foundError == 0 && $DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}

# do the 2 nodes belong to the same cluster?
sub checkClusterIdentity {
  printHeading("Checking cluster identity...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $hostnameCommand |") ) {
      print "Can't run $hostnameCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      while (<FILERDATA>) {
        chomp;
        if ($DEBUG) { print; printNewLine(1); }
        $hostName[$fn] = $_;
      }
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $cfPartnerCommand |") ) {
      print "Can't run $cfPartnerCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      while (<FILERDATA>) {
        chomp;
        if ($DEBUG) { print; printNewLine(1); }
        $partnerName[$fn] = $_;
      }
    }
    if ($DEBUG) { printNewLine(1); }
  }
  if ($partnerName[0] ne $hostName[1] || $partnerName[1] ne $hostName[0])
  {
    print "$hostName[0] and $hostName[1] are not members of the same cluster";
    printNewLine(1);
    return 1;
  }
  if ($DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}

# is cf enabled?
sub checkCfStatus {
  $foundError = 0;
  printHeading("Checking cf status...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $cfStatusCommand |") ) {
      print "Can't run $cfStatusCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      while (<FILERDATA>) {
        if ($DEBUG) { print; printNewLine(1); }
        if (m/Cluster disabled\./) {
          $foundError = 1;
        }
      }
    }
    close FILERDATA;
    if ($DEBUG) { printNewLine(1); }
  }
  if ($foundError == 0) {
    if ($DEBUG) {
      print "OK"; printNewLine(1);
    }
  }
  else {
    print "WARNING: Operator has not enabled the cluster.";
          printNewLine(1);
  }
  return 0;
}

# is fcp cfmode set the same?
sub checkFCPcfmodeMismatch {
  $foundError = 0;
  #for ($fn = 0; $fn < 2; $fn++) {
  #  if (!open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $licenseCommand|") ) {
  #    print "Can't run $licenseCommand on $filer[$fn]";
  #    printNewLine(1);
  #    return 1;
  #  }
  #  while (<FILERDATA>) {
  #    if(m/options.license.fcp=off/) {
  #	 return 0;
  #    }
  #  }
  #}
  printHeading("Checking fcp cfmode settings...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (!open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $fcpcfModeCommand 2>/dev/null |") ) {
      print "Can't run $fcpcfModeCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      $fcpcfmodestring[$fn] = <FILERDATA>;
      close FILERDATA;
      if ($fcpcfmodestring[$fn] !~ /fcp show cfmode/) {
        return 0;
      }
      if ($DEBUG) { print $fcpcfmodestring[$fn]; printNewLine(1); }
    }
  }
  close FILERDATA;
  if ($fcpcfmodestring[0] ne $fcpcfmodestring[1]) {
    $foundError = 1;
  }
  if ($foundError == 0) {
    if ($DEBUG) {
      print "OK"; printNewLine(1);
    }
  }
  else {
    print "FCP cfmode mismatch on cluster."; printNewLine(1);
  }
  return 0;
}


# look for options mismatch.
sub checkOptionsMismatch {
  $foundError = 0;
  printHeading("Checking options...");
  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    if (! open(FILERDATA, "$rshCommand[$fn] $filer[$fn] $optionsCommand |") ) {
      print "Can't run $optionsCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      $listIndex = 0;
      while (<FILERDATA>) {
	if (m/same/) {
	  s/\(.*\)//;
	  if ($DEBUG) { print; printNewLine(1);}
	  if ($fn == 0) {
	    $optionsList[$listIndex++] = $_;
	  }
	  else {
	    $listIndex = 0;
	    $foundOption = 0;
	    while ($listIndex <= $#optionsList &&  $foundOption == 0) {
	      if ($optionsList[$listIndex] eq $_) {
		$optionsList[$listIndex] = "ERASED";
		$foundOption = 1;
	      }
	      $listIndex++;
	    }
	    if ($foundOption == 0) {
	      $foundError = 1;
	      print "Option $_ in $filer[1] has no match in $filer[0].";
              printNewLine(1);
	    }
	  }
	}
      }
    }
    close FILERDATA;
    if ($DEBUG) { printNewLine(1); }
  }
  for ($listIndex = 0; $listIndex <= $#optionsList; $listIndex++) {
    if ($optionsList[$listIndex] ne "ERASED") {
      $foundError = 1;
      print "Option $optionsList[$listIndex] in $filer[0] has no match in $filer[1].";
      printNewLine(1);
    }
  }
  if ($foundError == 0 && $DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}


# load an array of two hashes with ifconfig data 
# keyed by ifname from the two nodes.
# call subs to do the real work of checking 
# for config errors
sub checkNetworkConfig {
  $foundError = 0;
  printHeading("Checking network interfaces...");
  for ($fn = 0; $fn < 2; $fn++) {
    if (! open (FILERDATA, "$rshCommand[$fn] $filer[$fn] $ifconfigCommand |") ) {
      print "Can't run $ifconfigCommand on $filer[$fn]";
      printNewLine(1);
      return 1;
    }
    else {
      $currentInterface = "";
      while (<FILERDATA>) {
        chomp;
        s/status\.if\.//;
#       if ($DEBUG) { print; printNewLine(1); }
        s/^(\w+)\.(.*)/($2)/;
        if ($1 ne $currentInterface) {
          if ($currentInterface ne "") {
            $hashArr[$fn]{$currentInterface} = $ifInfo;
          }
          $currentInterface = $1;
          $ifInfo = "";
        }
        $ifInfo .= "$2 ";
      }
      $hashArr[$fn]{$currentInterface} = $ifInfo;
      close FILERDATA;
    }
#   if ($DEBUG) { printNewLine(1); }
  }

  for ($fn = 0; $fn < 2; $fn++) {
    if ($DEBUG) {
      print "$filer[$fn] :-";
      printNewLine(1);
    }
    foreach (keys(%{$hashArr[$fn]})) {
      $addr = "";
      $subnet = "";
      if ($hashArr[$fn]{$_} =~ /^addr=([^, ]*),?[^, ]*,?([^,; ]*).*$/) {
	$addr = $1;
	$subnet = $2;
      }
      $mediatype = "";
      if ($hashArr[$fn]{$_} =~ /^.*mediatype=([^ ]*).*$/) {
	$mediatype = $1;
      }
      $ipspace = "default-ipspace";
      if ($hashArr[$fn]{$_} =~ /^.*ipspace=([^ ]*).*$/) {
	$ipspace = $1;
      }
      $partner = "";
      $partneraddr = "";
      if ($hashArr[$fn]{$_} =~ /^.*partner=([^ ]*).*partneraddr=([^ ]*).*$/) {
	$partner = $1;
	$partneraddr = $2;
      }
      $trunked = "";
      if ($hashArr[$fn]{$_} =~ /^.*trunked=([^ ]*).*$/) {
	$trunked = $1;
      }
      $type = "";
      if ($hashArr[$fn]{$_} =~ /^.*\stype=([^ ]*).*$/) {
	$type = $1;
      }
      $nfo = "";
      if ($hashArr[$fn]{$_} =~ /^.*nfo=([^ ]*).*$/) {
        $nfo = $1;
      }
      if ($DEBUG) {
	print "$_ -- addr=$addr subnet=$subnet mediatype=$mediatype ipspace=$ipspace ";
	print "partner=$partner partneraddr=$partneraddr trunked=$trunked type=$type nfo=$nfo";
	printNewLine(1);
      }

      # we don't care about trunked interfaces and the loopback inteface
      # (addr = 127.0.0.1) because they don't have partner interfaces.
      if (($trunked eq "" || $trunked eq "off") && ($addr ne "127.0.0.1")) {
	if (!($mediatype =~ /^OC.*$/)) {
	  if ($addr eq "" && $partner eq "" && $partneraddr eq "") {
	    # interface is not configured ... not considered an error condition
	    # $foundError++;
	    # print "Interface $_ on $filer[$fn] is not configured, no local address or partner assigned.";
	    # printNewLine(2);
	  }
	  else {
	    $foundError += checkInterface($_,$fn);
	  }
	}
	else {
	  $foundError += checkATMInterface($_,$fn);
	}
      }
    }
    printNewLine(1);
  }

  if ($foundError == 0 && $DEBUG) {
    print "OK"; printNewLine(1);
  }
  return 0;
}

# Given an ATM interface (not trunked) that is configured and 
# node to which the interface belongs, check whether it is configured
# correctly w.r.t. its partner.
sub checkATMInterface
{
# print "$_[0] is an ATM interface, logic to check this is TBD";
# printNewLine(2);
  return 0;
}

# Given an interface (not trunked) that is configured and 
# node to which the interface belongs, check whether it is configured
# correctly w.r.t. its partner.
sub checkInterface
{
  local($foundError) = 0;
  $localInterface = $_[0];
  $localIndex = $_[1];
  $partnerIndex = 1 - $localIndex;

  # every local interface that has a local address assigned must have
  # a unique matching partner interface on partner node.

  if ($addr ne "") {
    $retval = findPartnerInterface($partnerIndex, $localInterface, $addr);
    if ($retval eq "") {
      $foundError = 1;
      print "$localInterface on $filer[$localIndex] will not be taken over because it does not have a matching interface on partner node $filer[$partnerIndex]";
      printNewLine(1);
    }
    else {

      # local and partner interfaces should have same subnet, ipspace, mediatype, type and nfo values

      if ($hashArr[$partnerIndex]{$retval} =~ /^addr=[^, ]*,?[^, ]*,?([^,; ]*).*mediatype=([^ ]*).*$/) {
        if ($1 ne "" && $subnet ne $1) {
          $foundError = 1;
          print "$localInterface on $filer[$localIndex] may not be taken over correctly because it is on subnet $subnet, whereas partner interface $retval on $filer[$partnerIndex] is on subnet $1";
	  printNewLine(1);
        }
        if ($mediatype ne $2) {
          $foundError = 1;
          print "$localInterface on $filer[$localIndex] may not be taken over correctly because it has mediatype $mediatype, whereas partner interface $retval on $filer[$partnerIndex] has mediatype $2";
	  printNewLine(1);
        }        
      }
      if ($hashArr[$partnerIndex]{$retval} =~ /^.*ipspace=([^ ]*).*$/) {
        if ($ipspace ne $1) {
          $foundError = 1;
          print "$localInterface on $filer[$localIndex] may not be taken over correctly because it belongs to ipspace $ipspace, whereas partner interface $retval on $filer[$partnerIndex] belongs to ipspace $1";
	  printNewLine(1);
        } 
      }
      if ($hashArr[$partnerIndex]{$retval} =~ /^.*nfo=([^ ]*).*$/) {
        if ($nfo ne $1) {
          $foundError = 1;
          print "$localInterface on $filer[$localIndex] may not be taken over correctly because it has nfo option set to $nfo, whereas partner interface $retval on $filer[$partnerIndex] has nfo option set to $1";
          printNewLine(1);
        }
      }
      if ($hashArr[$partnerIndex]{$retval} =~ /^.*\stype=([^ ]*).*$/) {
	if ($type ne $1) {
	  $foundError = 1;
	  print "$localInterface on $filer[$localIndex] may not be taken over correctly because it is of type $type, whereas partner interface $retval on $filer[$partnerIndex] is of type $1";
	  printNewLine(1);
	}
      }
      else {
	if ($type ne "") {
	  $foundError = 1;
	  print "$localInterface on $filer[$localIndex] may not be taken over correctly because it is of type $type, whereas partner interface $retval on $filer[$partnerIndex] is of type ";
	  printNewLine(1);
	}
      }
    }
  }

  # every local interface that has a partner interface/address assigned 
  # must have a unique matching local interface/address on partner node

  if ($partner ne "" || ($partneraddr ne "" && $partneraddr ne "0.0.0.0")) {
    $retval = findLocalInterface($partnerIndex, $partner, $partneraddr);
    if ($retval eq "") {
      $foundError = 1;
      print "Takeover of $partner $partneraddr will fail because this interface does not exist on partner node $filer[$partnerIndex]";
      printNewLine(1);
    }
  }

  if ($foundError) {
    printNewLine(1);
  }
  return $foundError;

}

# Given the index into hashArr, and an interface and an addr
# find unique matching partner interface/address
sub findPartnerInterface
{
  $found = 0;
  foreach (keys(%{$hashArr[$_[0]]})) {
    $hashArr[$_[0]]{$_} =~ /^.*partner=([^ ]*).*partneraddr=([^ ]*).*$/;
    if ($1 eq $_[1]) {
      if ($2 ne "" && $2 ne "0.0.0.0" && $2 ne $_[2]) {
        print "$_ on $filer[$_[0]] is the partner interface for $1 on $filer[1-$_[0]], but the partner address $2 and local address $_[2] are not identical.";
	printNewLine(1);
        return "";
      }
      $found++;
      $retval = $_;
    }
    elsif ($2 eq $_[2]) {
      if ($1 ne "" && $1 ne $_[1]) {
        print "$_ on $filer[$_[0]] has partner address $2, which corresponds to local address $_[2] for interface $_[1] on $filer[1-$_[0]]; but the partner interface $1 in $_ and local interface $_[1] are not identical.";
	printNewLine(1);
        return "";
      }
      $found++;
      $retval = $_;
    }
  }
  if ($found == 0) {
    return "";
  }
  elsif ($found == 1) {
    return $retval;
  }
  else {
    print "There is more than 1 partner interface on $filer[$_[0]] for local interface $_[1] on $filer[1-$_[0]].";
    printNewLine(1);
    return "";
  }
}

# Given the index into hashArr, and an interface and an addr
# find unique matching local interface/address
sub findLocalInterface
{
  $found = 0;
  foreach (keys(%{$hashArr[$_[0]]})) {
    if ($_[1] ne "") {
      if ($_ eq $_[1]) {
        if ($_[2] ne "" && $_[2] ne "0.0.0.0" && $hashArr[$_[0]]{$_} =~ /^addr=([^, ]*).*$/) {
          if ($_[2] ne $1) {
            print "Local address $1 for interface $_[1] on partner $filer[$_[0]] is not same as partner address $_[2] for partner interface $_ on $filer[1-$_[0]]";
	    printNewLine(1);
            return "";
          }
        }
        $found++;
        $retval = $_;
      }
    }
    else {
      if ($hashArr[$_[0]]{$_} =~ /^addr=([^, ]*).*$/) {
        if ($_[2] eq $1) {
          $found++;
          $retval = $_;
        }
      }
    }
  }
  if ($found == 0) {
    return "";
  }
  elsif ($found == 1) {
    return $retval;
  }
  else {
    print "More than 1 interface has local address $_[2] on partner";
    printNewLine(1);
    return "";
  }
}

# print new line chars
sub printNewLine
{
  local($i);
  for ($i = 0; $i < $_[0]; $i++) {
    if ($ActAsCgi) {
      print "<BR>";
    }
    else {
      print "\n";
    }
  }
}

sub printHeading
{
  if ($DEBUG) {
    if ($ActAsCgi) {
      print "<H3>$_[0]</H3>";
    }
    else {
      print "\n$_[0]\n\n";
    }
  }
}

sub printUsage
{
  if (! $ActAsCgi) {
    print "Usage: $0 <name/IP addr of 1st node> <name/IP addr of 2nd node> [verbose]\n";
#
# print statement for 5.x removed
# Version < 7.x N/A for IBM N-series filer
#
  }
}

sub printAbout
{

  print "<H1>About Data ONTAP&#153 Cluster Configuration Checker</H1>
<H3>Copyright &copy; 2001 Network Appliance, Inc. All rights reserved.</H3>
<H3>Version 1.1</H3>
This tool was developed by Network Appliance.<BR>
It works on IBM N-series clusters running Data Ontap&#153 7.x";
}


sub printHelp
{
  print "<H1>Help for Data ONTAP&#153 Cluster Configuration Checker</H1>
Cluster Configuration Checker is a tool that detects errors
in the configuration of clustered Data ONTAP filers.<BR> It uses rsh to 
talk to the partners of the cluster.
<P>
Errors:
<LI>
If services licensed on the two nodes are not identical,
some services may be not be available in takeover mode.</LI>
<LI>
If options that should be identical in a cluster are not identical,
some options may be altered in takeover mode.</LI>
<LI>
If network interfaces are wrongly configured, takeover of 
these interfaces will fail causing clients that access them
to lose connectivity in takeover mode.</LI>
<P>
Warnings:
<LI>
OS versions in the two clustered nodes are not identical.</LI>
<LI>
Clustering is licensed, but takeover capability is disabled.</LI>
<P>
Sometimes rsh commands issued to a filer do not complete.<BR>
This may happen because a filer is down or is not reachable from
the client that is running this script.<BR>
Click your browser's Stop button to recover.
";
}
